import sys, os, inspect, time
from optparse import OptionParser
from poaupdater import uSysDB, uConfig, uLogging, uHCL

class nodeInfo:
	pass

if __name__ == "__main__":
	global opts

	opt = OptionParser(usage = "Usage: %prog [options] <cluster id>\n")
	opt.add_option("-v", "--verbose", action="store_true", help="if specified, will cause detailed trace output to log", default=False)
	opt.add_option("-F", "--fix", action="store_true", help="erases superfluous components from Web nodes of given NG UI cluster and reconfigures Apache service accordingly", default=False)

	clusterID = None
	opts, args = opt.parse_args()
	if args:
		try:
			clusterID = int(args[0])
		except:
			pass
	if clusterID is None:
		opt.print_help()
		sys.exit(1)

	config = uConfig.Config(verbose = opts.verbose)
	uLogging.init(config)

	webNodeInfos = []
	con = uSysDB.connect()
	cur = con.cursor()
	cur.execute("SELECT w.host_id, h.primary_name FROM apache_h2e_nodes AS w INNER JOIN hosts AS h USING (host_id) WHERE w.cluster_id=%d AND w.type='W' ORDER BY w.server_weight DESC" % (clusterID,))

	for wnode in cur.fetchall():
		sys.stdout.write("----- Checking availability of NG UI cluster Web node #%d '%s'...\n" % (wnode[0], wnode[1]))
		ni = nodeInfo()
		ni.hostId = wnode[0]
		ni.hostName = wnode[1]
		ni.redisRPM = uHCL.runHCLCmd(ni.hostId, """ rpm -q redis || echo -n """).strip()
		if ni.redisRPM:
			sys.stdout.write("----- Web node has %s installed\n" % (ni.redisRPM,))
		webNodeInfos.append(ni)

	if opts.fix:
		currentDirPath = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
		apacheConfigFile = "/etc/httpd/conf/httpd.conf"
		sourceConfigFile = currentDirPath+"/no-redis-httpd.conf"
		targetConfigFile = "/etc/httpd/conf/no-redis-httpd.conf"

		processedWebNodes = []

		for ni in webNodeInfos:
			rhcl = uHCL.Request(host_id=ni.hostId, user='root', group='root')
			rhcl.transfer('1', sourceConfigFile, "/etc/httpd/conf/") #1 is MN host ID here
			rhcl.perform()
			uHCL.runHCLCmd(ni.hostId, """ sed -i "s|#_HOST_ID_#|%d|" "%s" """ % (ni.hostId, targetConfigFile))
			uHCL.runHCLCmd(ni.hostId, """ mv -f -S .PAPERF-2287 "%s" "%s" """ % (targetConfigFile, apacheConfigFile))
			try:
				uHCL.runHCLCmd(ni.hostId, """ service httpd restart """)
			except:
				processedWebNodes.append(ni)
				for nr in reversed(processedWebNodes):
					uHCL.runHCLCmd(nr.hostId, """ mv -f "%s.PAPERF-2287" "%s" """ % (apacheConfigFile, apacheConfigFile))
					uHCL.runHCLCmd(nr.hostId, """ service httpd restart """)
				raise

			processedWebNodes.append(ni)
		for ni in webNodeInfos:
			if ni.redisRPM:
				uHCL.runHCLCmd(ni.hostId, """ yum -y erase redis """)
		cur.execute("UPDATE apache_h2e_clusters SET cluster_type=2 WHERE cluster_id=%d" % clusterID)
		con.commit()
	else:
		con.rollback()
